@@ -6,7 +6,7 @@ from abc import ABC, abstractmethod
 
 from .stopping_criterion import SufficientProgressCriterion
 
-from .utils.safe_import import set_benchmark
+from .utils.safe_import import set_benchmark_module
 from .utils.dynamic_modules import get_file_hash
 from .utils.dynamic_modules import _reconstruct_class
 
@@ -20,8 +20,8 @@ class BaseSolver(ParametrizedNameMixin, DependenciesMixin, ABC):
     Solvers that derive from this class should implement three methods:
 
     - ``set_objective(self, **objective_parameters)``: prepares the solver to
-      be called on a given problem. ``**objective_parameters`` are the output
-      of the method ``to_dict`` from the benchmark objective. In particular,
+      be called on a given problem. ``**objective_parameters`` is the output of
+      the method ``get_objective`` from the benchmark objective. In particular,
       this method should dumps the parameter to compute the objective function
       in a file for command line solvers to reduce the impact of dumping the
       data to the disk in the benchmark.
@@ -43,6 +43,9 @@ class BaseSolver(ParametrizedNameMixin, DependenciesMixin, ABC):
       logarithmically to get more an more precise points.
     - ``'tolerance'``: call the run method with tolerance deacreasing
       logarithmically to get more and more precise points.
+    - ``'callback'``: a callable that should be called after each iteration or
+      epoch. This callable periodically calls the objective's `compute`
+      and returns False when the solver should stop.
 
     """
 
@@ -83,7 +86,19 @@ class BaseSolver(ParametrizedNameMixin, DependenciesMixin, ABC):
             If skip is False, the reason should be None.
         """
         self._objective = objective
-        objective_dict = objective.to_dict()
+        # XXX remove in version 1.4
+        objective_dict = objective.get_objective()
+        if objective_dict is None:
+            assert hasattr(objective, "to_dict"), (
+                "Objective needs to implement `get_objective` that returns "
+                "a dictionary to be passed to `set_objective`"
+            )
+            warnings.warn(
+                "The method ``Objective.to_dict`` has been deprecated in "
+                "favor of ``Objective.get_objective``. Using it will no "
+                "longer work in version 1.4", FutureWarning
+            )
+            objective_dict = objective.to_dict()
 
         # Check if the objective is compatible with the solver
         skip, reason = self.skip(**objective_dict)
@@ -100,8 +115,8 @@ class BaseSolver(ParametrizedNameMixin, DependenciesMixin, ABC):
         Parameters
         ----------
         **objective_parameters : dict
-            Dictionary obtained as the output of the method ``to_dict`` from
-            the benchmark ``Objective``.
+            Dictionary obtained as the output of the method ``get_objective``
+            from the benchmark ``Objective``.
         """
         ...
 
@@ -148,8 +163,8 @@ class BaseSolver(ParametrizedNameMixin, DependenciesMixin, ABC):
         Parameters
         ----------
         **objective_parameters : dict
-            Dictionary obtained as the output of the method ``to_dict`` from
-            the benchmark ``Objective``.
+            Dictionary obtained as the output of the method ``get_objective``
+            from the benchmark ``Objective``.
 
         Returns
         -------
@@ -171,7 +186,7 @@ class BaseSolver(ParametrizedNameMixin, DependenciesMixin, ABC):
     @staticmethod
     def _reconstruct(module_filename, parameters, objective,
                      pickled_module_hash=None, benchmark_dir=None):
-        set_benchmark(benchmark_dir)
+        set_benchmark_module(benchmark_dir)
         Solver = _reconstruct_class(
             module_filename, 'Solver', benchmark_dir, pickled_module_hash,
         )
@@ -261,7 +276,7 @@ class BaseDataset(ParametrizedNameMixin, DependenciesMixin, ABC):
     @staticmethod
     def _reconstruct(module_filename, pickled_module_hash, parameters,
                      benchmark_dir):
-        set_benchmark(benchmark_dir)
+        set_benchmark_module(benchmark_dir)
         Dataset = _reconstruct_class(
             module_filename, 'Dataset', benchmark_dir, pickled_module_hash,
         )
@@ -284,7 +299,7 @@ class BaseObjective(ParametrizedNameMixin, DependenciesMixin):
     - `set_data(**data)`: stores the info from a given dataset to be able to
       compute the objective value on these data.
 
-    - `to_dict()`: exports the data from the dataset as well as the parameters
+    - `get_objective()`: exports the data from the dataset and the parameters
       from the objective function as a dictionary that will be passed as
       parameters of the solver's `set_objective` method in order to specify the
       objective function of the benchmark.
@@ -313,8 +328,9 @@ class BaseObjective(ParametrizedNameMixin, DependenciesMixin):
         """
         ...
 
-    @abstractmethod
-    def to_dict(self):
+    # to uncomment in version 1.4, once `to_dict` has been deprecated.
+    # @abstractmethod
+    def get_objective(self):
         """Return the objective parameters for the solver.
 
         Returns
@@ -383,7 +399,24 @@ class BaseObjective(ParametrizedNameMixin, DependenciesMixin):
         if skip:
             return skip, reason
 
+        # Check if parameters are modified by set_data
+        parameters = {}
+        for key in self._parameters:
+            parameters[key] = getattr(self, key)
         self.set_data(**data)
+        for key in self._parameters:
+            has_changed = parameters[key] != getattr(self, key)
+            if hasattr(has_changed, '__iter__'):
+                has_changed = any(has_changed)
+
+            if has_changed:
+                raise ValueError(
+                    f"Parameter {key} has been changed from {parameters[key]} "
+                    f"to {getattr(self, key)}. "
+                    "Parameters of Objective should not be "
+                    "modified by 'set_data'."
+                )
+
         return False,  None
 
     def skip(self, **data):
@@ -407,7 +440,7 @@ class BaseObjective(ParametrizedNameMixin, DependenciesMixin):
         return False, None
 
     def get_one_solution(self):
-        """Return one point in which the objective function can be evaluated.
+        """Return one solution for which the objective can be evaluated.
 
         This method is mainly for testing purposes, to check that the return
         computation and the return type of `Objective.compute`. It should
@@ -441,7 +474,7 @@ class BaseObjective(ParametrizedNameMixin, DependenciesMixin):
     @staticmethod
     def _reconstruct(module_filename, pickled_module_hash, parameters,
                      dataset, benchmark_dir):
-        set_benchmark(benchmark_dir)
+        set_benchmark_module(benchmark_dir)
         Objective = _reconstruct_class(
             module_filename, 'Objective', benchmark_dir, pickled_module_hash,
         )
