@@ -66,6 +66,11 @@ def test_dataset_get_data(benchmark, dataset_class):
     if dataset_class.name.lower() == 'finance':
         pytest.skip("Do not download finance.")
 
+    # XXX TODO remove when scikit-learn releases the fix
+    # see https://github.com/scikit-learn/scikit-learn/pull/23358
+    if dataset_class.name.lower() == 'leukemia':
+        pytest.skip("Leukemia download is broken in scikit-learn 1.1.0")
+
     data = dataset._get_data()
     assert isinstance(data, (tuple, dict)), (
         "Output of get_data should be a 2-tuple or a dict."
@@ -221,10 +226,12 @@ def _test_solver_one_objective(solver, objective):
     beta_hat = solver.get_result()
     objective(beta_hat)
 
-    if is_convex:
+    # Only check optimality or convex problems, with simple enough return type
+    if is_convex and isinstance(beta_hat, np.ndarray):
         val_star = objective(beta_hat)['objective_value']
         for _ in range(100):
             eps = 1e-5 * np.random.randn(*beta_hat.shape)
             val_eps = objective(beta_hat + eps)['objective_value']
+
             diff = val_eps - val_star
             assert diff >= 0
