@@ -2,15 +2,17 @@
 
 set -e
 
-. activate $CONDAENV
+conda info
+pip freeze
 
-TEST_CMD="python -m pytest -vs --showlocals --durations=20 --junitxml=$JUNITXML --pyargs"
-TEST_CMD="$TEST_CMD --test-env $CONDAENV --pyargs"
+TEST_CMD="python -m pytest -vs --showlocals --durations=20 --junitxml=$JUNIT_XML --pyargs"
+TEST_CMD="$TEST_CMD --test-env $CONDA_ENV --pyargs"
 
 # Un-comment when debugging the CI
 # TEST_CMD="$TEST_CMD --skip-install"
 
 if [[ "$COVERAGE" == "true" ]]; then
+    export COVERAGE_PROCESS_START=".coveragerc"
     TEST_CMD="$TEST_CMD --cov=benchopt --cov-config=.coveragerc"
     python continuous_integration/install_coverage_subprocess_pth.py
 fi
